﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Data.Objects;
using bookstore.Data;
using bookstore.Data.DTOs;

namespace bookstore.Services
{
    /// <summary>
    /// Summary description for Bookstore
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class Bookstore : System.Web.Services.WebService
    {
        private ObjectContext objectContext;
        private ObjectSet<Book> bookSet;
        private ObjectSet<Author> authorSet;
        private ObjectSet<User> userSet;

        public Bookstore() : this(new bookstoreContainer()) { }

        public Bookstore(ObjectContext objectContext)
        {
            this.objectContext = objectContext;
            this.bookSet = objectContext.CreateObjectSet<Book>();
            this.authorSet = objectContext.CreateObjectSet<Author>();
            this.userSet = objectContext.CreateObjectSet<User>();
        }

        [WebMethod]
        public string HelloWorld()
        {
            return "Hello World";
        }

        [WebMethod]
        public List<Book> GetBooks()
        {
            return bookSet.ToList<Book>();
        }

        [WebMethod]
        public void AddBook(string title, double price, int authorId)
        {
            Book book = new Book();
            book.Price = price;
            book.Title = title;

            Author author = authorSet.Where(a => a.Id == authorId).FirstOrDefault();

            book.Authors.Add(author);

            objectContext.SaveChanges();
        }

        [WebMethod]
        public void BuyBook(int bookId, int userId)
        {
            Book book = bookSet.Where(b => b.Id == bookId).FirstOrDefault();
            User user = userSet.Where(u => u.Id == userId).FirstOrDefault();

            user.Books.Add(book);

            objectContext.SaveChanges();
        }

        /// <summary>
        /// DTO-based example
        /// </summary>
        [WebMethod]
        public List<BookWithAuthors> GetBooksWithAuthors()
        {
            var q = from b in bookSet
                    from a in b.Authors
                    where b.Authors.Contains(a)
                    orderby a.LastName, a.FirstName, b.Title
                    select new BookWithAuthors()
                    {
                        id = b.Id,
                        title = b.Title,
                        price = b.Price,
                        author = a.FirstName + " " + a.LastName
                    };

            return q.ToList<BookWithAuthors>();
        }
    }
}
